"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateRoomData = exports.addUserToRoomMembers = exports.disActiveRoom = exports.addUserToDislikeList = exports.createRoom = exports.getOneRoom = exports.getRooms = void 0;
const Room_1 = require("../models/Room");
const getRooms = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const userId = req.user._id.toString();
    const userName = req.user.name;
    const rooms = yield Room_1.Room.find()
        .or([
        { type: "public" },
        {
            members: { $elemMatch: { id: userId } },
        },
    ])
        .and([{ isActive: true }, { dislikeMembers: { $nin: [userId] } }]);
    res.json({ rooms: rooms });
});
exports.getRooms = getRooms;
const getOneRoom = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const userId = req.user._id.toString();
    const roomId = req.params.id;
    const room = yield Room_1.Room.findOne({ _id: roomId });
    if (!room) {
        return res.status(404).json({
            error: "no room",
        });
    }
    res.status(200).json({ room: room });
});
exports.getOneRoom = getOneRoom;
const createRoom = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const userId = req.user._id.toString();
    const userName = req.user.name;
    const { name, type } = req.body;
    const room = yield new Room_1.Room({
        name,
        type: type ? type : "public",
        isActive: true,
        members: [{ name: userName, id: userId }],
        createdBy: { name: userName, id: userId },
        dislikeMembers: [],
    });
    yield room.save();
    yield (0, exports.getRooms)(req, res);
});
exports.createRoom = createRoom;
const addUserToDislikeList = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const roomId = req.body.id;
    if (!roomId) {
        return res.status(400).json({
            message: "Invalid room id",
        });
    }
    let room;
    try {
        room = yield Room_1.Room.findOne({
            _id: roomId,
        });
    }
    catch (error) {
        return res.status(400).json({
            message: "Room not found",
        });
    }
    if (!room) {
        return res.status(400).json({
            message: "Room not found",
        });
    }
    room.dislikeMembers.push(req.user.id);
    yield room.save();
    yield (0, exports.getRooms)(req, res);
});
exports.addUserToDislikeList = addUserToDislikeList;
const disActiveRoom = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const roomId = req.params.id;
    if (!roomId) {
        return res.status(400).json({
            message: "Invalid room id",
        });
    }
    let room;
    try {
        room = yield Room_1.Room.findOne({
            _id: roomId,
        });
    }
    catch (error) {
        return res.status(400).json({
            message: "Room not found",
        });
    }
    if (!room) {
        return res.status(404).json({
            message: "Room not found",
        });
    }
    room.isActive = false;
    yield room.save();
    yield (0, exports.getRooms)(req, res);
});
exports.disActiveRoom = disActiveRoom;
const addUserToRoomMembers = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const roomId = req.params.id;
    const { name, id: userId } = req.body;
    if (!roomId) {
        return res.status(400).json({
            message: "Invalid room id",
        });
    }
    let room;
    try {
        room = yield Room_1.Room.findOne({
            _id: roomId,
        });
    }
    catch (error) {
        return res.status(404).json({
            message: "Room not found",
        });
    }
    if (!room) {
        return res.status(404).json({
            message: "Room not found",
        });
    }
    // console.log(userId);
    const userInList = room.members.find((member) => {
        if (member.id === userId) {
            return true;
        }
        return;
    });
    if (userInList) {
        return res.status(302).json({
            message: "User just exist in Room",
        });
    }
    room.members.push({ name, id: userId });
    yield room.save();
    yield (0, exports.getRooms)(req, res);
});
exports.addUserToRoomMembers = addUserToRoomMembers;
const updateRoomData = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const roomId = req.params.id;
    const data = req.body;
    let room;
    if (!roomId) {
        return res.status(404).json({
            error: "Invalid room id",
        });
    }
    try {
        room = yield Room_1.Room.findOne().and([
            { _id: roomId },
            // {
            //   createdBy: { $elemMatch: { id: req.user._id } },
            // },
        ]);
    }
    catch (error) {
        return res.status(500).json({
            error: "Room not found",
        });
    }
    if (!room) {
        return res.status(404).json({
            error: "Room not found",
        });
    }
    room.name = data.name;
    room.type = data.type;
    room.members = data.members;
    yield room.save();
    res.send("ok");
});
exports.updateRoomData = updateRoomData;
//# sourceMappingURL=roomsController.js.map