"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteFileFromCloud = exports.sortMessagesByDate = exports.getLastMessagesFromRoom = exports.updateUserNewMessages = exports.socketService = void 0;
const index_1 = require("../index");
// import { cloudinary } from "../index2";
const Message_1 = require("../models/Message");
const Room_1 = require("../models/Room");
const User_1 = require("../models/User");
const message_1 = require("../types/message");
const user_dto_1 = require("../types/user.dto");
require("dotenv").config();
const socketioJwt = require("socketio-jwt");
const socketService = (io) => __awaiter(void 0, void 0, void 0, function* () {
    // io.use(
    //   socketioJwt.authorize({
    //     secret: config.ACCESS_TOKEN_KEY,
    //     handshake: true,
    //     auth_header_required: true,
    //   })
    // );
    yield io.on("connection", (socket) => {
        // console.log("connected");
        socket.on("new-user", () => __awaiter(void 0, void 0, void 0, function* () {
            socket.emit("new-user", "działa");
        }));
        socket.on("join-room", (room) => __awaiter(void 0, void 0, void 0, function* () {
            socket.join(room);
            let roomMessages = yield (0, exports.getLastMessagesFromRoom)(room);
            roomMessages = (0, exports.sortMessagesByDate)(roomMessages);
            socket.emit("room-messages", roomMessages);
        }));
        socket.on("delete-room-messages", (messageId, room) => __awaiter(void 0, void 0, void 0, function* () {
            // socket.join(room);
            let message;
            try {
                message = yield Message_1.Message.findOne({
                    _id: messageId,
                });
            }
            catch (error) {
                return;
            }
            if (!message)
                return;
            message.isActive = false;
            yield message.save();
            deleteFileFromCloud(message);
            let roomMessages = yield (0, exports.getLastMessagesFromRoom)(room);
            roomMessages = (0, exports.sortMessagesByDate)(roomMessages);
            socket.emit("room-messages", roomMessages);
        }));
        socket.on("message-room", (FECurrentRoomId, content, contentType = message_1.MessageTypes.text, sender, time, date) => __awaiter(void 0, void 0, void 0, function* () {
            const newMessage = yield new Message_1.Message({
                content,
                contentType,
                date,
                isActive: true,
                from: sender,
                sockedId: "",
                time,
                to: FECurrentRoomId,
            });
            yield newMessage.save();
            let roomMessages = yield (0, exports.getLastMessagesFromRoom)(FECurrentRoomId);
            const sortedRoomMessages = (0, exports.sortMessagesByDate)(roomMessages);
            io.to(FECurrentRoomId).emit("room-messages", sortedRoomMessages);
            // send notification to frontend
            socket.broadcast.emit("notifications", FECurrentRoomId);
            // console.log(socket.handshake.query.token);
            // send notification to database
            if (String(FECurrentRoomId).includes("-")) {
                const roomMembers = FECurrentRoomId.split("-");
                roomMembers.forEach((member) => {
                    if (member === sender.id) {
                        return;
                    }
                    else {
                        (0, exports.updateUserNewMessages)(member, FECurrentRoomId);
                    }
                });
            }
            else {
                const messageRoom = yield Room_1.Room.findOne({ _id: FECurrentRoomId });
                if (!messageRoom) {
                    return;
                }
                messageRoom.members.forEach((member) => {
                    if (member.id === sender.id) {
                        return;
                    }
                    else {
                        (0, exports.updateUserNewMessages)(member.id, FECurrentRoomId);
                    }
                });
            }
        }));
        socket.on("user-status", ({ userId, status }) => __awaiter(void 0, void 0, void 0, function* () {
            const user = yield User_1.User.findOne({
                _id: userId,
            });
            if (!user) {
                console.log("user-status not found");
                return;
            }
            user.status =
                status === user_dto_1.UserStatus.online ? user_dto_1.UserStatus.online : user_dto_1.UserStatus.offline;
            try {
                yield user.save();
            }
            catch (error) {
            }
            finally {
            }
        }));
    });
});
exports.socketService = socketService;
const updateUserNewMessages = (memberID, FECurrentRoomId) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const updateUser = yield User_1.User.findOne({ _id: memberID });
    if (!updateUser)
        return;
    let newMessages = (_a = updateUser.newMessages) !== null && _a !== void 0 ? _a : {};
    if (newMessages[FECurrentRoomId]) {
        newMessages[FECurrentRoomId] += 1;
    }
    else {
        newMessages[FECurrentRoomId] = 1;
    }
    updateUser.newMessages = newMessages;
    try {
        updateUser.markModified("newMessages");
        yield updateUser.save();
    }
    catch (error) { }
});
exports.updateUserNewMessages = updateUserNewMessages;
const getLastMessagesFromRoom = (room) => __awaiter(void 0, void 0, void 0, function* () {
    let roomMessages = yield Message_1.Message.aggregate([
        { $match: { to: room } },
        { $match: { isActive: true } },
        { $group: { _id: "$date", messagesByDate: { $push: "$$ROOT" } } },
    ]);
    return roomMessages;
});
exports.getLastMessagesFromRoom = getLastMessagesFromRoom;
const sortMessagesByDate = (messages) => {
    return messages.sort((a, b) => {
        let date1 = a._id.split("/");
        let date2 = b._id.split("/");
        let date11 = Number(date1[0]) * 1 +
            Number(date1[1]) * 1000 +
            Number(date1[2]) * 100000;
        let date22 = Number(date2[0]) * 1 +
            Number(date2[1]) * 1000 +
            Number(date2[2]) * 100000;
        return date11 < date22 ? -1 : 1;
    });
};
exports.sortMessagesByDate = sortMessagesByDate;
function deleteFileFromCloud(message) {
    var _a;
    try {
        if (["image", "video", "raw"].includes(String(message.contentType))) {
            const cloudUrlLink = (_a = message.content) === null || _a === void 0 ? void 0 : _a.split(" | ")[0];
            // http://res.cloudinary.com/dkdynfku8/video/upload/v1674412502/messages/63c2ccc179f9d8ef641a5596-63c2ccb279f9d8ef641a5592/abc.mp4.mp4
            const slashIndex = cloudUrlLink === null || cloudUrlLink === void 0 ? void 0 : cloudUrlLink.lastIndexOf("/messages");
            const dotIndex = cloudUrlLink === null || cloudUrlLink === void 0 ? void 0 : cloudUrlLink.lastIndexOf(".");
            const publicID = cloudUrlLink === null || cloudUrlLink === void 0 ? void 0 : cloudUrlLink.substring(slashIndex + 1, dotIndex);
            index_1.cloudinary.uploader
                .destroy(publicID, {
                invalidate: true,
                resource_type: `${message.contentType}`,
            })
                .then((result) => {
                console.log(result);
            })
                .catch((error) => {
                console.log(error);
            });
        }
    }
    catch (error) {
        console.log(error);
        return;
    }
}
exports.deleteFileFromCloud = deleteFileFromCloud;
//# sourceMappingURL=socketService.js.map