"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.io = exports.cloudinary = void 0;
const body_parser_1 = require("body-parser");
// import { v2 as cloudinary } from "cloudinary";
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
require("express-async-errors");
const express_handlebars_1 = require("express-handlebars");
const express_rate_limit_1 = __importDefault(require("express-rate-limit"));
const http_1 = require("http");
const path_1 = __importDefault(require("path"));
const socket_io_1 = require("socket.io");
const config_1 = require("./config/config");
const db_1 = require("./config/db");
const socketService_1 = require("./controllers/socketService");
const messagesRouter_1 = require("./routes/messagesRouter");
const roomsRouter_1 = require("./routes/roomsRouter");
const userRouter_1 = require("./routes/userRouter");
require("dotenv").config();
const PORT = 3001;
(0, db_1.connectDB)();
const corsOptions = {
    origin: [config_1.config.corsOrigin],
    credentials: true,
    optionSuccessStatus: 200,
};
const limiter = (0, express_rate_limit_1.default)({
    windowMs: 1000,
    max: 100,
});
const app = (0, express_1.default)();
const httpServer = (0, http_1.createServer)(app);
app.use((0, cors_1.default)(corsOptions));
app.use((0, cookie_parser_1.default)());
app.use(limiter);
app.use(express_1.default.json());
app.use(express_1.default.static("public"));
app.use((0, body_parser_1.urlencoded)({
    extended: true,
}));
//routing
app.use("/api/users", userRouter_1.userRouter);
app.use("/api/rooms", roomsRouter_1.roomsRouter);
app.use("/api/messages", messagesRouter_1.messagesRouter);
// handlebars
const hbsPath = path_1.default.join(__dirname, "./views");
app.engine("handlebars", (0, express_handlebars_1.engine)());
app.set("view engine", "handlebars");
app.set("views", hbsPath);
//Cloudinary
exports.cloudinary = require("cloudinary").v2;
exports.cloudinary.config({
    cloud_name: config_1.config.cld_cloud_name,
    api_key: config_1.config.cld_api_key,
    api_secret: config_1.config.cld_secret_key,
    secure: true,
});
//error middleware
// app.use(errorHandler);
app.use(function (req, res, next) {
    res.setHeader("Access-Control-Allow-Origin", config_1.config.corsOrigin);
    next();
});
httpServer.listen(PORT, "0.0.0.0");
exports.io = new socket_io_1.Server(httpServer, {
    cors: {
        origin: [config_1.config.corsOrigin],
        credentials: true,
        methods: ["GET", "POST"],
    },
    transports: ["websocket", "polling"],
});
// socket.io service
(0, socketService_1.socketService)(exports.io);
//# sourceMappingURL=index.js.map